"""
Utility methods for working with the Spotify API
"""
from math import ceil
from pathlib import Path
from typing import Any, Dict, Iterable, Protocol

import yaml
from spotify_model import Paging
from spotipy import Spotify, SpotifyClientCredentials

DEFAULT_LIMIT = 50


def read_credentials(path: Path) -> Spotify:
    "Read credentials from a YAML file and construct a Spotify client"

    with open(path, "r") as credentials_file:
        credentials = yaml.safe_load(credentials_file)
        credentials_manager = SpotifyClientCredentials(credentials["client_id"], credentials["client_secret"])
        return Spotify(client_credentials_manager=credentials_manager)


class Paginated(Protocol):
    "Protocol definition for functions that will be provided to the `exhaust` handler"

    def __call__(self, limit: int, offset: int) -> Paging:
        ...


def exhaust(function: Paginated, limit: int = DEFAULT_LIMIT) -> Iterable[Dict[str, Any]]:
    "Exhaust a function that returns a pagination object"
    response = function(limit=limit, offset=0)

    for item in response.items:
        yield item

    for i in range(1, ceil(response.total / limit)):
        response = function(limit=response.limit, offset=response.limit * i)
        for item in response.items:
            yield item
